REBOL [
	Title: "REBOL/View Desktop - Network Connections"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2003"
	Tabs: "Tabspace is 4. PLEASE KEEP IT THAT WAY."
	License: {
		This software is the property of REBOL Technologies and is
		licensed to you under the terms of the REBOL/View Desktop
		License (included in this distribution).
	}
]

connecting: none  ; in the process of connecting

connect-view: has [info] [
	; Connect the desktop to the Internet and check the rebol.com site for updates.
	if connecting [connecting: false exit]
	connecting: true
	show-status "connecting"
	show-progress/no-status 100 1
	dbug ["Connected?" connected?]
	either all [
		connected?
		any [find [socks socks5 generic] system/schemes/default/proxy/type find-site "www.rebol.com"]
		info: read-thru/progress/update/expand http://www.rebol.com/view/update.r :show-progress
	][
		dbug "Online"
		show-progress 1 1
		alive?: true
		clear index-cache
		if error? try [do to-string info][
			alert "Error on connection. Please report this to feedback@rebol.com. Thank you."
		]
		show-main-status reform ["Connected to the Internet at" now/time]
		show-status ""
	][
		dbug "Offline"
		show-progress 1 1
		show-main-status {Cannot connect. Working offline.}
		tell {Click on "local" to connect.}
		show-status "local"
		alive?: false
	]
]

find-site: func [host [string!] /local ip dport n secs] [
	; Find IP address of the specified host by doing an async DNS lookup.
	; Do not return until the request has completed or an error happened.
	; GUI events will still be processed (because WAIT is called), but this
	; is NOT the central call to the WAIT function. Avoid nesting WAIT.
	; This is an old way of doing an asynchronous DNS lookup.
	if error? try [dport: open/no-wait dns:///async][
		return read to-url rejoin ["dns://" host] ; fallback for some machines
	]
	insert dport host
	n: 0
	secs: any [user-prefs/connect-wait 60]
	while [error? try [ip: copy dport]] [
		wait [dport 1]
		n: n + 1
		show-progress/no-status secs n
		if any [not connecting n >= secs] [ip: none break]
	]
	connecting: false
	close dport
	ip
]


