REBOL [
	Title: "REBOL/View Desktop - Emailer"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2003"
	Tabs: "Tabspace is 4. PLEASE KEEP IT THAT WAY."
	License: {
		This software is the property of REBOL Technologies and is
		licensed to you under the terms of the REBOL/View Desktop
		License (included in this distribution).
	}
]

alive?: true

ctx-emailer: context [
	f-to: f-cc: f-email: f-subject: f-msg: none

	;-- Emailer layout:
	lo: [
		style tx text bold 80x24 right
		style btn button 80x24
		style fld field 300x24
		across origin 6x6 space 2x1
		tx "To:"      f-to: fld return
		tx "CC:"      f-cc: fld return
		tx "From:"    f-email: info 300 para [] return
		tx "Subject:" f-subject: fld return
		tx "Message:" f-msg: area wrap 300x200 return
		below at f-msg/offset + f-msg/size * 0x1 + 5x-82 space 0x4
		btn "Send" #"^S" [submit-email]
		btn "Clear" [clear-all]
		btn "Close" escape [close-em]
	]

	clear-all: does [
		; Clear emailer fields:
		clear-fields lo
		f-email/text: form system/user/email
		clear f-msg/text
		f-msg/line-list: none
		show lo
		focus f-to
	]

	submit-email: has [sending user hdr subject] [
		; Send the email:
		sending: flash "Sending..."
		either error? try [
			if empty? f-to/text [error-out-here]
			user: load/all f-to/text
			if not empty? f-cc/text [append user load/all f-cc/text]
			hdr: make system/standard/email [subject: f-subject/text]
			send/header user f-msg/text hdr
		][
			unview/only sending
			alert "Error sending email. Check fields and check your network setup."
		][
			unview/only sending
			close-em
			alert "Your email has been sent."
		]
	]

	close-em: does [
		; Close the emailer view:
		unview/only lo
	]

	set 'emailer func [
		"Pops up a quick email sender."
		/to "Specify a target address"
		target [string! email!]
		/subject "Specify a subject line"
		what [string!]
		/local req
	][
		if block? lo [lo: layout lo  center-face lo]
		if not alive? [
			alert "Email cannot be sent when offline."
			exit
		]
		if not all [system/user/email system/schemes/default/host] [
			req: request [{Your email settings are missing from the network preferences.
				Set them now?} "Setup" "Ignore" "Cancel"]
			if none? req [exit]
			if req [set-user]
		]
		clear-all
		if to [f-to/text: copy target]
		if subject [f-subject/text: copy what]
		focus f-to
		view/new/title lo "Emailer"
	]
]

;emailer do-events