REBOL [
	Title: "REBOL/View Desktop - Icon Handling"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2003"
	Tabs: "Tabspace is 4. PLEASE KEEP IT THAT WAY."
	License: {
		This software is the property of REBOL Technologies and is
		licensed to you under the terms of the REBOL/View Desktop
		License (included in this distribution).
	}
]

;=== Icon Functions =========================================================

icon-space: 6		; spacing between icons (could be made a preference someday)
dtf-slide-max: 10
image-cache: []		; block of desktop image files loaded this session

make-icon: func [ico /local icon][
	; Make an icon from the default icon face.
	; Set its color and load its image, using a default image based on the
	; icon file suffix (type).
	icon: make-face dtc-icon
	icon/user-data: ico
	icon/text: ico/name
	if ico/color [icon/font: make icon/font [color: ico/color]]

	icon/pane: make icon/pane [
		image: either ico/image [
			any [load-safe-image ico/image default-icon]
		][
			switch/default ico/type [
				folder folder-icon
				link [load-safe-image view-root/desktop/icons/html.gif]
			][
				load-default-icon ico/item
			]
		]
		feel: none
		color: none
		effect: any [ico/effect [key 174.154.122 shadow smooth]]
	]

	; Make icon proper size:
	if icon/pane/image [
		icon/pane/size: 64x48
		icon/pane/offset: 0x0
		if not outside? icon/pane/size icon/pane/image/size [
			icon/pane/size: icon/pane/image/size
			icon/pane/offset: 64x48 - icon/pane/image/size / 2
			if not ico/edge [
				icon/pane/edge/effect: none
				icon/pane/edge/size: none
			]
		]
	]

	icon
]

load-safe-image: func [
	; Load an icon from a file or URL without choking on it:
	file [file! url!] "Local file or remote URL"
	/update "Force update from source site"
	/clear "Purge the entire cache"
	/local image
][
	if clear [system/words/clear image-cache recycle]
	file: full-path file
	if any [update  not image: select image-cache file] [
		if all [update image] [remove/part find image-cache file 2]
		image: load-binary file not update
		repend image-cache [file image]
	]
	all [image? image image] ; returns file data
]

to-icon-path: func [name [word!]] [
	; Convert a file's datatype (word) to a loadable image name:
	join view-root/desktop/icons ["/" name ".gif"]
]

load-default-icon: func [file /local n][
	; Load the default icon for a file:
	if all [
		email? file
		n: load-safe-image view-root/desktop/icons/email.gif
	][return n]
	if all [
		file? file
		n: map-suffix file
		n: to-icon-path n
		not error? try [n: load n]
	][return n]
	default-icon
]

layout-icons: function [icons] [xy is][
	; Layout icons in a pane:
	sort/compare icons func [a b] [(form a/text) < (form b/text)] ;>
	xy: dtc-icon/offset
	is: 0
	foreach icon icons [
		if icon/style = 'icon [
			if (first xy + icon/size) > dtc-face/size/x [
				xy: (xy + icon/size + icon-space * 0x1) + (dtc-icon/offset * 1x0)
			]
			icon/offset: xy
			xy: icon/size * 1x0 + xy + (icon-space * 1x0)
			is: icon/size/y
		]
	]
	dtc-face/size/y: dtf-slide-max: xy/y + is
	dtf-slide/data: 0.0
	dtc-face/offset/y: 0
	remove find dtf-face/pane dtf-slide
	if dtf-slide-max > dtf-icons/size/y [
		append dtf-face/pane dtf-slide
		dtf-slide/redrag dtf-icons/size/y / dtf-slide-max
	]
]

dtc-icon/feel: make dtc-icon/feel [
	; Specify the event processing used for icons:

	over: func [f a pos of /local str][
		either a [ 
			f/font/color: dtw-skin/over-text-color
			if f/user-data/color [f/font/color: complement f/user-data/color]
			if f/user-data/over-color [f/font/color: f/user-data/over-color]
			show f
			f/font/color: any [f/user-data/color dtw-skin/base-text-color black] ;white
			if f/user-data/info [tell f/user-data/info]
		][
			show f
			tell ""
		]
	]

	engage: func [face action event /local icon][
		icon: face/user-data
		switch action [
			down [face/state: on]
			alt-down [face/state: on]
			up   [
				if face/state [
					if none? icon/type [icon/type: 'file] ; default
					dbug ["Icon do" third icon]
					switch icon/type [
						file   [do-file icon/item any [icon/stats true]]
						link   [if icon/item [browse full-path icon/item]]
						folder [show-folder icon/item]
					]
				]
				face/state: off
			]
			alt-up [
				dbug ["Icon info" third icon]
				if all [face/state icon] [show-icon-info icon]
				face/state: off
			]
			over [face/state: on]
			away [face/state: off]
		]
		cue face action
		show face
	]
]

