REBOL [
	Title: "REBOL/View Desktop - Index Files"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2003"
	Tabs: "Tabspace is 4. PLEASE KEEP IT THAT WAY."
	License: {
		This software is the property of REBOL Technologies and is
		licensed to you under the terms of the REBOL/View Desktop
		License (included in this distribution).
	}
]

index-cache: []		; block of index files loaded this session

;-- Desktop Objects ----------------------------------------------------------

reb-folder: context [
	title:		; folder title
	summary:	; folder summary
	updated:
	expires:
	notice:
	image:		; folder backdrop image
	color:		; folder backdrop color
	effect:		; folder backdrop effect
	text-color:	; main text color
	over-color:	; text over color
	icons: none
]

reb-icon: context [
	type:		; 'file, 'folder, 'email
	item:		; file!, url!, email!
	name:		; "icon label"
	color:		; icon text color
	over-color: ; icon's over color
	stats:		; [size date]
	info:		; icon caption
	image:		; custom icon image
	edge:		; default edge
	effect:		; special icon effect
	folder:		; sub-folder object
		none
]

;-- Folder Index File Processing ---------------------------------------------

load-index: func [path /local page here] [
	; Load and parse a folder index file through the index cache:
	dbug ["Loading index" path]
	page: read-binary path here: found? find index-cache path
	if none? page [return none]
	if not here [append index-cache path]
	if error? try [page: load/all to-string page][return none]
	parse-index page ; return the index folder
]

index-file?: func [blk] [
	; Determine if a block is a valid folder index file:
	all [ ; REBOL [type: 'index] required
		blk/1 = 'rebol
		block? blk/2
		'index = select blk/2 to-set-word 'type
	]
]

remove-draw: func [eff /local p] [
	; Remove draw effects, they are not secure:
	eff: copy eff
	while [p: find eff 'draw] [remove/part p 2]
	eff
]

parse-index: func [spec /local val f i] [
	; Parse a folder index file. This is the official specification
	; for the format of the index file. Return a folder object.
	if not index-file? spec [return none]
	f: make reb-folder [icons: copy []]
	parse spec [
		2 skip ; skip REBOL header
		any [ ;h: (probe first h)
			  'title   set val string! (f/title: val)
			| 'summary set val string! (f/summary: val)
			| 'updated set val date!   (f/updated: val)
			| 'expires set val date!   (f/expires: val)
			| 'notice  set val string! (f/notice: val)
			| 'text-color set val tuple! (f/text-color: val)
				[set val tuple! (f/over-color: val) | none]
			| 'backdrop [
				some [
					  set val 'tile (f/effect: [tile])
					| set val [file! | url!] (f/image: val)
					| set val path! (if 'view-root = first :val [f/image: val])
					| set val tuple! (f/color: val)
					| set val block! (f/effect: remove-draw val)
				]
			]
		]
		any [ ;h: ( probe copy/part h 5)
			(i: make reb-icon [color: f/text-color over-color: f/over-color])
			set val ['file | 'folder | 'link | 'service] (i/type: val)
			some [
				  set val [file! | url! | email! | 'goto | 'help | 'console | 'quit] (i/item: val)
				| set val string! (i/name: val)
				| set val tuple!  (i/color: val)
				| set val into [integer! [date! | none]] (i/stats: val)
				| set val path! (if 'view-root = first :val [i/item: val])
				| 'info set val string! (i/info: val)
				| 'edge (i/edge: true)
				| 'icon set val [file! | url! | word! | path!] (
					if all [path? :val 'view-root = first :val][val: val]
					i/image: either word? val [to-icon-path val][val]
				)
				| 'effect set val block! (i/effect: remove-draw val)
			]
			(if none? i/name [i/name: to-string i/item] append f/icons i)
			| none skip
		]
		| end
	]
	;probe f
	f ; returns a folder
]
