REBOL [
	Title: "REBOL/View Desktop - Main Module"
	Version: 1.2.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2003"
	Tabs: "Tabspace is 4. PLEASE KEEP IT THAT WAY."
	License: {
		This software is the property of REBOL Technologies and is
		licensed to you under the terms of the REBOL/View Desktop
		License (included in this distribution).
	}
]

test-mode: on
link?: false

;-- Variables ----------------------------------------------------------------

view-root: what-dir	; base directory for file references
alive?: false		; true when connected to net


;-- Utility Functions --------------------------------------------------------

dbug: func [msg [block! any-string!] /local dbg] [
	if not dbg: user-prefs/debug [exit]
	msg: head insert remold msg "desktop "
	either file? dbg [write/append dbg msg][print msg]
]

;-- Missing Modules ----------------------------------------------------------

view-prefs: does [alert "No user settings or prefs panel."]
set-user: does [view-prefs]

;-- View Desktop Modules -----------------------------------------------------

#include %../sdk/source/view.r
#include %../sdk/source/prot.r

#include %vt-initfiles.r
#include %vt-prefs.r
#include %vt-images.r
#include %vt-window.r
#include %vt-folder.r
#include %vt-icon.r
#include %vt-iconinfo.r
#include %vt-initfiles.r
#include %vt-index.r
#include %vt-launch.r
#include %vt-goto.r
#include %vt-help.r
#include %vt-emailer.r
#include %vt-editor.r
#include %vt-connect.r

;-- Initialization -----------------------------------------------------------

init-desktop: has [product] [
	dbug ["Desktop boot" now]

	load-prefs
	windowize-dtw
	resize-dtw
	reskin-dtw dtw-skin
	view/new/options/title dtw-face [resize] "Desktop"
	detect-dtw
	show-id none
	show-modules
	show-welcome
	init-desktop-files
	load-file-types
	show-main-status either user-prefs/auto-connect [
		"Connecting to the Internet..."
	][
		{Working offline. Click "local" to connect.}
	]
	show-services view-root/desktop/services.r
	if user-prefs/auto-connect [connect-view]
	show-bookmarks view-root/desktop/bookmarks.r
]

desktop: does [
	; Function that is called to start the desktop.
;	if block? ctx-viewtop [ctx-viewtop: context ctx-viewtop]
;	ctx-viewtop/init-desktop
	init-desktop
	do-events
]

if test-mode [do desktop]

none ; (Return value to REBOL component system: no callback)
